@@ -220,6 +220,9 @@ static gboolean g_idle_dispatch    (GSource     *source,
 G_LOCK_DEFINE_STATIC (main_loop);
 static GMainContext *default_main_context;
 
+#if defined(G_PLATFORM_WIN32) && defined(__GNUC__)
+__declspec(dllexport)
+#endif
 GSourceFuncs g_timeout_funcs =
 {
   g_timeout_prepare,
@@ -228,6 +231,9 @@ GSourceFuncs g_timeout_funcs =
   NULL
 };
 
+#if defined(G_PLATFORM_WIN32) && defined(__GNUC__)
+__declspec(dllexport)
+#endif
 GSourceFuncs g_idle_funcs =
 {
   g_idle_prepare,
@@ -504,7 +510,7 @@ g_poll (GPollFD *fds,
 
 /**
  * g_main_context_ref:
- * @loop: a #GMainContext
+ * @context: a #GMainContext
  * 
  * Increases the reference count on a #GMainContext object by one.
  **/
@@ -569,7 +575,7 @@ g_main_context_unref_and_unlock (GMainContext *context)
 
 /**
  * g_main_context_unref:
- * @loop: a #GMainContext
+ * @context: a #GMainContext
  * 
  * Decreases the reference count on a #GMainContext object by one. If
  * the result is zero, free the context and free all associated memory.
