@@ -27,22 +27,19 @@
 #ifndef __GMODULE_H__
 #define __GMODULE_H__
 
-extern const char      *g_log_domain_gmodule;
 #include <glib.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
+G_BEGIN_DECLS
 
 /* exporting and importing functions, this is special cased
  * to feature Windows dll stubs.
  */
 #define	G_MODULE_IMPORT		extern
-#if defined (G_OS_WIN32)
+#ifdef G_PLATFORM_WIN32
 #  define	G_MODULE_EXPORT		__declspec(dllexport)
-#else /* !G_OS_WIN32 */
+#else /* !G_PLATFORM_WIN32 */
 #  define	G_MODULE_EXPORT
-#endif /* !G_OS_WIN32 */
+#endif /* !G_PLATFORM_WIN32 */
 
 typedef enum
 {
@@ -58,26 +55,25 @@ typedef void	     (*GModuleUnload)	 (GModule	*module);
 gboolean	g_module_supported	   (void) G_GNUC_CONST;
 
 /* open a module `file_name' and return handle, which is NULL on error */
-GModule*	g_module_open		   (const gchar		*file_name,
-					    GModuleFlags	 flags);
+GModule*              g_module_open          (const gchar  *file_name,
+					      GModuleFlags  flags);
 
 /* close a previously opened module, returns TRUE on success */
-gboolean	g_module_close		   (GModule		*module);
+gboolean              g_module_close         (GModule      *module);
 
 /* make a module resident so g_module_close on it will be ignored */
-void		g_module_make_resident	   (GModule		*module);
+void                  g_module_make_resident (GModule      *module);
 
 /* query the last module error as a string */
-gchar*		g_module_error		   (void);
+G_CONST_RETURN gchar* g_module_error         (void);
 
 /* retrive a symbol pointer from `module', returns TRUE on success */
-gboolean	g_module_symbol		   (GModule		*module,
-					    const gchar		*symbol_name,
-					    gpointer		*symbol);
+gboolean              g_module_symbol        (GModule      *module,
+					      const gchar  *symbol_name,
+					      gpointer     *symbol);
 
 /* retrive the file name from an existing module */
-gchar*		g_module_name		   (GModule		*module);
-
+G_CONST_RETURN gchar* g_module_name          (GModule      *module);
 
 /* Build the actual file name containing a module. `directory' is the
  * directory where the module file is supposed to be, or NULL or empty
@@ -90,12 +86,10 @@ gchar*		g_module_name		   (GModule		*module);
  *
  * No checks are made that the file exists, or is of correct type.
  */
-gchar*		g_module_build_path	  (const gchar		*directory,
-					   const gchar		*module_name);
+gchar*                g_module_build_path    (const gchar  *directory,
+					      const gchar  *module_name);
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
 
+G_END_DECLS
 
 #endif /* __GMODULE_H__ */
